package ru.dbotthepony.mc.otm.matter

import ru.dbotthepony.mc.otm.core.ImpreciseFraction

interface IMatterValue : Comparable<IMatterValue> {
	val matter: ImpreciseFraction
	val complexity: Double

	val hasMatterValue: Boolean get() = matter.isPositive && complexity > 0.0

	operator fun plus(other: IMatterValue): IMatterValue {
		return MatterValue(matter + other.matter, complexity + other.complexity)
	}

	operator fun minus(other: IMatterValue): IMatterValue {
		return MatterValue(matter - other.matter, complexity - other.complexity)
	}

	override fun compareTo(other: IMatterValue): Int {
		val matterComparison = matter.compareTo(other.matter)

		if (matterComparison == 0) {
			return complexity.compareTo(other.complexity)
		}

		return matterComparison
	}

	/**
	 * ZERO
	 */
	companion object : IMatterValue {
		override val matter: ImpreciseFraction
			get() = ImpreciseFraction.ZERO
		override val complexity: Double
			get() = 0.0
	}
}

data class MatterValue(
	override val matter: ImpreciseFraction,
	override val complexity: Double
) : IMatterValue
